/*
 * The MIT License
 * Copyright © 2014-2019 Ilkka Seppälä
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.iluwatar.leaderfollowers;

import org.junit.Assert;
import org.junit.Test;

/**
 * Tests for WorkCenter
 */
public class WorkCenterTest {

  @Test
  public void testCreateWorkers() {
    var taskSet = new TaskSet();
    var taskHandler = new TaskHandler();
    var workCenter = new WorkCenter();
    workCenter.createWorkers(5, taskSet, taskHandler);
    Assert.assertEquals(workCenter.getWorkers().size(), 5);
    Assert.assertEquals(workCenter.getWorkers().get(0), workCenter.getLeader());
  }

  @Test
  public void testNullLeader() {
    var workCenter = new WorkCenter();
    workCenter.promoteLeader();
    Assert.assertNull(workCenter.getLeader());
  }

  @Test
  public void testPromoteLeader() {
    var taskSet = new TaskSet();
    var taskHandler = new TaskHandler();
    var workCenter = new WorkCenter();
    workCenter.createWorkers(5, taskSet, taskHandler);
    workCenter.removeWorker(workCenter.getLeader());
    workCenter.promoteLeader();
    Assert.assertEquals(workCenter.getWorkers().size(), 4);
    Assert.assertEquals(workCenter.getWorkers().get(0), workCenter.getLeader());
  }
}
